==> musashi-1.0.0.tar.gz: musashi-1.0.0/cmd/xtsep.c <==
/*============================================================================*/
/* ѹ                                                                   */
/*----------------------------------------------------------------------------*/
/* 1.0 : APIб 2003/06/20                                           */
/*============================================================================*/

#include <musashi.h>
#include <stdlib.h>
#include <string.h>

#include <xtsepHelp.h>
struct mssComHelp comHelp={
  "xtsep",        /* ޥ̾       */
  "1.0",          /* С       */
  HELPT,          /* ޥɥȥ */
  HELPS,          /*              */
  HELPE,          /*              */
  HELPR,          /* ȥޥ     */
  HELPA,          /* Ծ         */
  HELPB,          /* ХݡȾ */
  HELPH           /* ۡڡ     */
};

extern struct mssGlobalVariables mssGV;

/*ѥƹ¤*/
struct Path {
  char *str[50];
  struct mssFields *flds;
  char *endStr;
  int cnt;
  char *optStr; /*ץλѴ*/
};

/*------------------------------------------------------------------------------
̾bufꤷ̾μΥݥ󥿤֤
$\0Ѵ
ex)
str=$Customer/aaa/$(Date).gz
buf="Customer"
end="/aaa/$(Date).gz"
------------------------------------------------------------------------------*/
char *setFldName(char *buf,char *str){
  char *end;
  int i=0;

  *str='\0'; /* $\0Ѵ */

  str++;
  if(*str=='('){
    end=mssSkipStrBlock(str,'(',')');
    *(end-1)='\0';
    strcpy(buf,str+1);
  }else{
    end=str;
    while(*end!='/' && *end!='\0'){
      if(i>=MssFieldNameMaxLen){
        mssShowErrMsg("too long field name on -d");
        mssEnd(mssErrorNoDefault);
      }
      *(buf+i)=*end;
      end++; i++;
    }
    *(buf+i)='\0';
  }
  return(end);
}

/*------------------------------------------------------------------------------
-dץ󤫤Path¤Τ˥åȤ롣
ex)
-d /tmp/$Customer/aaa/$(Date).gz
p->str[0]="/tmp/"   ,   p->fld[0]=$Cusotmer
p->str[1]="/aaa/"   ,   p->fld[1]=$Date
p->endStr=".gz"
p->cnt=2
p->optStr="Customer,Date"
------------------------------------------------------------------------------*/
struct Path *setPath(char *path, struct mssHeader *hd){
  struct Path *p;
  char buf[MssFieldNameMaxLen];
  char opt[1024];
  char *str;

  p=mssCalloc(sizeof(struct Path),"setOptFld");
  p->flds=mssInitFields();
  p->cnt=0;
  opt[0]='\0';

  str=path;
  while(*path!='\0'){
    if(*path=='$'){
      if(p->cnt>=50){
        mssShowErrMsg("too many field names on -d");
        mssEnd(mssErrorNoDefault);
      }

      /*ܤΥå*/
      path=setFldName(buf,path);
      strcat(opt,buf); strcat(opt,",");
      mssAddFieldsByFldInfo(p->flds,mssFldNam2Add(hd->flds,buf));
      //addFieldbyHDstr(p->flds,hd,buf);

      /*ܤʸΥå*/
      p->str[p->cnt]=str;
      str=path;

      /*ȥå*/
      p->cnt++;
    }
  path++;
  }
  if(p->cnt==0){
    mssShowErrMsg("no field name on -d");
    mssEnd(mssErrorNoDefault);
  }
  p->endStr=str;

  opt[strlen(opt)-1]='\0';
  p->optStr=mssMalloc( sizeof(char)*(strlen(opt)+1),"setPath" );
  strcpy(p->optStr,opt);

  return(p);
}

/*------------------------------------------------------------------------------
Path¤ΤȼºݤΥǡ쥳ɤ顢¥ե̾ꤹ
------------------------------------------------------------------------------*/
void setPathName(
  char *pathName,
  struct Path *path,
  struct mssFldRecDbl *frd,
  MssOptKEY *optKey){

  int i;
  int len=1; /*1ʸ'\0'Τ*/

  *pathName='\0';
  for(i=0; i<path->cnt; i++){
    /*ե̾ĹΥå*/
    len+=strlen(path->str[i])+strlen(*frd->pnt[frd->new]);
    if(len>=MssFileNameMaxLen) {
      mssShowErrMsg("path name exceed 1024 charactors");
      mssEnd(mssErrorNoDefault);
    }

    /*ʸȥǡͤϢ뤷Ƥ*/
    strcat(pathName,path->str[i]);
    strcat(pathName,*(frd->pnt[frd->new]+MssFlds2num(path->flds,i)));
  }
  /*ǸʸϢ*/
  len+=strlen(path->endStr);
  if(len>=MssFileNameMaxLen){
    mssShowErrMsg("path name exceed 1024 charactors");
    mssEnd(mssErrorNoDefault);
  }
  strcat(pathName,path->endStr);
}

int main(int argc, char *argv[]){
/*============================================================================*/
/* ץ                                                       */
/*============================================================================*/
/*----------------------------------------------------------------------------*/
/* (Ūѡѥ᡼ǤϻǤʤ                          */
/*----------------------------------------------------------------------------*/
  MssOptKEY optKEY={
    OKEY,   /* ץ󥿥                                             */
    "k",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    MssFieldMaxCnt, /* ǽʺܿ                                 */
    "i",    /* оݤȤϥǡΥ(GUI)                  */
    2,      /* ǥե(Υץ󤬻ꤵʤäȤư) */
            /* 1:ƤιԤۤ륭ͤȤư                             */
            /* 2:ƤιԤƱͤȤư)                            */
  };

/*----------------------------------------------------------------------------*/
/* ϥե̾                                                             */
/*----------------------------------------------------------------------------*/
  MssOptSTR optDIR={
    OSTR,   /* ץ󥿥                                             */
    "d",    /* (ʣʸԲ)                                   */
    1,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    NULL  , /* ǥե                                                   */
    1,      /* ʸκǾĹ                                               */
    MssFileNameMaxLen,  /* ʸκĹ                                   */
    DIRT,   /* ΥץΥȥ(Helpɽ)                         */
    DIRC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ǥ쥯ȥ                                                       */
/*----------------------------------------------------------------------------*/
  MssOptFLG optFRC={
    OFLG,   /* ץ󥿥                                             */
    "p",    /* (ʣʸԲ)                                   */
    0,      /* ǥե(Ūˤ0) onˤȤ1ˤ          */
    FRCT,   /* ΥץΥȥ(Helpɽ)                         */
    FRCC    /* ΥץΥ(Helpɽ)                         */
  };


/*----------------------------------------------------------------------------*/
/* ϥե                                                               */
/*----------------------------------------------------------------------------*/
  MssOptINF optINF={
    OINF,   /* ץ󥿥                                             */
    "i",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ                                         */
    1,      /* ǽκե                                     */
    0,      /*1:file not foundΥ顼ǽλʤ 0:                   */
    INFT,   /* ΥץΥȥ(Helpɽ)                         */
    INFC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ̽                                                                   */
/*----------------------------------------------------------------------------*/
  MssOptFLG optZIP={
    OFLG,   /* ץ󥿥                                             */
    "z",    /* (ʣʸԲ)                                   */
    0,      /* ǥե(Ūˤ0) onˤȤ1ˤ          */
    ZIPT,   /* ΥץΥȥ(Helpɽ)                         */
    ZIPC    /* ΥץΥ(Helpɽ)                         */
  };
    
/*----------------------------------------------------------------------------*/
/* plain text                                                                 */
/*----------------------------------------------------------------------------*/
  MssOptFLG optTXT={
    OFLG,   /* ץ󥿥                                             */
    "t",    /* (ʣʸԲ)                                   */
    0,      /* ǥե(Ūˤ0) onˤȤ1ˤ          */
    TXTT,   /* ΥץΥȥ(Helpɽ)                         */
    TXTC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* եѥǥ쥯ȥ̾                                             */
/*----------------------------------------------------------------------------*/
  MssOptSTR optTMP={
    OSTR,   /* ץ󥿥                                             */
    "T",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    MssTempDir, /* ǥե                                               */
    1,      /* ʸκǾĹ                                               */
    MssFileNameMaxLen,  /* ʸκĹ                                   */
    TMPT,   /* ΥץΥȥ(Helpɽ)                         */
    TMPC    /* ΥץΥ(Helpɽ)                         */
  };
    
/*----------------------------------------------------------------------------*/
/* ץޤȤ                                                       */
/*----------------------------------------------------------------------------*/
  void *opt[]={&optDIR,&optFRC,&optINF,&optZIP,&optTXT,&optTMP,NULL};
    
/*============================================================================*/
/* ѿ                                                             */
/*============================================================================*/
  struct mssHeader    *hdi; /*ϥե<head>Ǽ¤*/
  struct mssHeader    *hdo; /*ϥե<head>Ǽ¤*/
  struct mssFPR       *fpr; /*ϥե빽¤                */
  struct mssFPW       *fpw=NULL; /*ϥե빽¤                */
  struct mssFldRecDbl *frd; /*-ԥХåե¤           */
  struct mssFields    *sf;  /*ȹܹ¤                  */
  int sorted;               /*Ⱥѥå                */

  struct Path *path;
  char pathName[MssFileNameMaxLen];
  int count;

/*----------------------------------------------------------------------------*/
/*                                                                      */
/*----------------------------------------------------------------------------*/
  mssInit(argc,argv,&comHelp);       /* ʥʤɤν              */
  mssHelpDoc(opt,&comHelp,argc,argv);/* إ                                */
  mssSetOption(opt,argc,argv);       /* ޥɥץ              */
  fpr=mssOpenFPR(optINF.str,4);      /* ϥե륪ץ                  */
  hdi=mssReadHeader(fpr);            /* إåɤ߹                      */

  path=setPath(optDIR.str,hdi);   /*path򥻥å*/

  /* optKey򤳤Ƕ*/
  optKEY.set=1;
  optKEY.str=mssStrdup(path->optStr);
  optKEY.cnt=0;
  optKEY.nam=mssTokByChr(optKEY.str,',',&optKEY.cnt,1); /*,ˤȡʬ*/
  optKEY.diffSame=0;
  mssSetOptKey(&optKEY, hdi);        /* -k ܤإåܤ˴ϢŤ     */

  /*ȹܤκ*/
  sf=mssInitFields();
  mssAddFieldsByFields(sf,optKEY.flds); /* -k ܤ򥽡ȹܤȤƥå    */
  mssSetFieldsSortPriority(sf);         /* ֹͥϿˤդ   */
  sorted=mssChkSorted(sf,hdi);          /* ȺѤå                 */

/*----------------------------------------------------------------------------*/
/*ϥإåκȽ                                                    */
/*----------------------------------------------------------------------------*/
  /*ϥإåν(ȥΥԡ)*/
  hdo=mssInitCpyHeader(hdi);
    
  /*ϥإåܤɲ*/
  mssAddFieldsByFields(hdo->flds,hdi->flds);                                
    
  /*ȤɬפʤsfΥȾȿ*/                        
  if(!sorted){
    mssSetFieldsSort(hdo->flds,sf);
  }

/*----------------------------------------------------------------------------*/
/*ᥤ롼                                                              */
/*----------------------------------------------------------------------------*/
  /*ȤɬѤʤХȤƥȺѤߥեȤƥץ*/
  if(!sorted)
    fpr=mssReopenFPRsort(fpr,4,sf,hdi->flds->cnt,optTMP.str);

  frd=mssInitFRD(hdi->flds->cnt);
  while( EOF!=mssReadFRD(fpr,frd) ){

    if(mssGV.inCnt==0){
      setPathName(pathName,path,frd,&optKEY);
      fpw=mssOpenFPW(pathName,optZIP.set,optFRC.set);
      mssWriteHeader(hdo, fpw);
    }

    /*֥쥤ν*/
    if(mssKeyBreak(frd, &optKEY)){
      mssWriteFooter(fpw);    /*եåν*/
      mssCloseFPW(fpw);                /*ϥեΥ*/
      if(frd->eof) break;
      setPathName(pathName,path,frd,&optKEY);
      fpw=mssOpenFPW(pathName,optZIP.set,optFRC.set);
      mssWriteHeader(hdo, fpw);
    }

    mssGV.inCnt++;
    count++;

    /*̾Ԥν*/
    mssWriteFld(frd->pnt[frd->new],frd->fldCnt,"\n",fpw);
    mssGV.outCnt++;
  }
  mssFreeFRD(frd);
  mssFreeFields(path->flds);
  mssFree(path->optStr);
  mssFree(path);
  mssFree(optKEY.str);
  mssFree(*optKEY.nam);
  mssFree(optKEY.nam);

/*----------------------------------------------------------------------------*/
/*եå&λ                                                       */
/*----------------------------------------------------------------------------*/
  mssCloseFPR(fpr);       /*ϥեΥ      */
  mssFreeFields(sf);      /* ȹܹ¤Τΰ賫 */
  mssFreeHeader(hdi);     /* ϥإåΰ賫         */
  mssFreeHeader(hdo);     /* ϥإåΰ賫         */
  mssFreeOption(opt);     /* ץΰ賫         */
  mssShowEndMsg();        /* λå             */
  mssEnd(mssExitSuccess); /* λ                       */
  return(0);              /* to avoid warning message   */
}

==> musashi-1.0.2.tar.gz: musashi-1.0.2/cmd/xtsep.c <==
/*============================================================================*/
/* ѹ                                                                   */
/*----------------------------------------------------------------------------*/
/* 1.0 : APIб 2003/06/20                                           */
/* 1.1 : -d $a$bΤ褦Ϣ³̾λǤԶ罤 2003/08/07         */
/* 1.2 : setFLdName̵¥롼׽  2003/08/07                               */
/*============================================================================*/

#include <musashi.h>
#include <stdlib.h>
#include <string.h>

#include <xtsepHelp.h>
struct mssComHelp comHelp={
  "xtsep",        /* ޥ̾       */
  "1.2",          /* С       */
  HELPT,          /* ޥɥȥ */
  HELPS,          /*              */
  HELPE,          /*            */
  HELPR,          /* ȥޥ     */
  HELPA,          /* Ծ         */
  HELPB,          /* ХݡȾ */
  HELPH           /* ۡڡ     */
};

extern struct mssGlobalVariables mssGV;

/*ѥƹ¤*/
struct Path {
  char *str[50];
  struct mssFields *flds;
  char *endStr;
  int cnt;
  char *optStr; /*ץλѴ*/
};

/*------------------------------------------------------------------------------
̾bufꤷ̾μΥݥ󥿤֤
$\0Ѵ
ex)
str=$Customer/aaa/$(Date).gz
buf="Customer"
end="/aaa/$(Date).gz"
------------------------------------------------------------------------------*/
char *setFldName(char *buf,char *str){
  char *end;
  int i=0;

  *str='\0'; /* $\0Ѵ */

  str++;
  if(*str=='('){
    end=mssSkipStrBlock(str,'(',')');
    *(end-1)='\0';
    strcpy(buf,str+1);
  }else{
    end=str;
    while(*end!='/' && *end!='\0' && *end!='$'){
      if(i>=MssFieldNameMaxLen){
        mssShowErrMsg("too long field name on -d");
        mssEnd(mssErrorNoDefault);
      }
      *(buf+i)=*end;
      end++; i++;
    }
    *(buf+i)='\0';
  }
  return(end);
}

/*------------------------------------------------------------------------------
-dץ󤫤Path¤Τ˥åȤ롣
ex)
-d /tmp/$Customer/aaa/$(Date).gz
p->str[0]="/tmp/"   ,   p->fld[0]=$Cusotmer
p->str[1]="/aaa/"   ,   p->fld[1]=$Date
p->endStr=".gz"
p->cnt=2
p->optStr="Customer,Date"
------------------------------------------------------------------------------*/
struct Path *setPath(char *path, struct mssHeader *hd){
  struct Path *p;
  char buf[MssFieldNameMaxLen];
  char opt[1024];
  char *str;

  p=mssCalloc(sizeof(struct Path),"setOptFld");
  p->flds=mssInitFields();
  p->cnt=0;
  opt[0]='\0';

  str=path;
  while(*path!='\0'){
    if(*path=='$'){
      if(p->cnt>=50){
        mssShowErrMsg("too many field names on -d");
        mssEnd(mssErrorNoDefault);
      }

      /*ܤΥå*/
      path=setFldName(buf,path);
      strcat(opt,buf); strcat(opt,",");
      mssAddFieldsByFldInfo(p->flds,mssFldNam2Add(hd->flds,buf));

      /*ܤʸΥå*/
      p->str[p->cnt]=str;
      str=path;

      /*ȥå*/
      p->cnt++;
    }else{
      path++;
    }
  }
  if(p->cnt==0){
    mssShowErrMsg("no field name on -d");
    mssEnd(mssErrorNoDefault);
  }
  p->endStr=str;

  opt[strlen(opt)-1]='\0';
  p->optStr=mssStrdup(opt);

  return(p);
}

/*------------------------------------------------------------------------------
Path¤ΤȼºݤΥǡ쥳ɤ顢¥ե̾ꤹ
------------------------------------------------------------------------------*/
void setPathName(
  char *pathName,
  struct Path *path,
  struct mssFldRecDbl *frd,
  MssOptKEY *optKey){

  int i;
  int len=1; /*1ʸ'\0'Τ*/

  *pathName='\0';
  for(i=0; i<path->cnt; i++){
    /*ե̾ĹΥå*/
    len+=strlen(path->str[i])+strlen(*frd->pnt[frd->new]);
    if(len>=MssFileNameMaxLen) {
      mssShowErrMsg("path name exceed 1024 charactors");
      mssEnd(mssErrorNoDefault);
    }

    /*ʸȥǡͤϢ뤷Ƥ*/
    strcat(pathName,path->str[i]);
    strcat(pathName,*(frd->pnt[frd->new]+MssFlds2num(path->flds,i)));
  }
  /*ǸʸϢ*/
  len+=strlen(path->endStr);
  if(len>=MssFileNameMaxLen){
    mssShowErrMsg("path name exceed 1024 charactors");
    mssEnd(mssErrorNoDefault);
  }
  strcat(pathName,path->endStr);
}

int main(int argc, char *argv[]){
/*============================================================================*/
/* ץ                                                       */
/*============================================================================*/
/*----------------------------------------------------------------------------*/
/* (Ūѡѥ᡼ǤϻǤʤ                          */
/*----------------------------------------------------------------------------*/
  MssOptKEY optKEY={
    OKEY,   /* ץ󥿥                                             */
    "k",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    MssFieldMaxCnt, /* ǽʺܿ                                 */
    "i",    /* оݤȤϥǡΥ(GUI)                  */
    2,      /* ǥե(Υץ󤬻ꤵʤäȤư) */
            /* 1:ƤιԤۤ륭ͤȤư                             */
            /* 2:ƤιԤƱͤȤư)                            */
  };

/*----------------------------------------------------------------------------*/
/* ϥե̾                                                             */
/*----------------------------------------------------------------------------*/
  MssOptSTR optDIR={
    OSTR,   /* ץ󥿥                                             */
    "d",    /* (ʣʸԲ)                                   */
    1,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    NULL  , /* ǥե                                                   */
    1,      /* ʸκǾĹ                                               */
    MssFileNameMaxLen,  /* ʸκĹ                                   */
    DIRT,   /* ΥץΥȥ(Helpɽ)                         */
    DIRC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ǥ쥯ȥ                                                       */
/*----------------------------------------------------------------------------*/
  MssOptFLG optFRC={
    OFLG,   /* ץ󥿥                                             */
    "p",    /* (ʣʸԲ)                                   */
    0,      /* ǥե(Ūˤ0) onˤȤ1ˤ          */
    FRCT,   /* ΥץΥȥ(Helpɽ)                         */
    FRCC    /* ΥץΥ(Helpɽ)                         */
  };


/*----------------------------------------------------------------------------*/
/* ϥե                                                               */
/*----------------------------------------------------------------------------*/
  MssOptINF optINF={
    OINF,   /* ץ󥿥                                             */
    "i",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ                                         */
    1,      /* ǽκե                                     */
    0,      /*1:file not foundΥ顼ǽλʤ 0:                   */
    INFT,   /* ΥץΥȥ(Helpɽ)                         */
    INFC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* ̽                                                                   */
/*----------------------------------------------------------------------------*/
  MssOptFLG optZIP={
    OFLG,   /* ץ󥿥                                             */
    "z",    /* (ʣʸԲ)                                   */
    0,      /* ǥե(Ūˤ0) onˤȤ1ˤ          */
    ZIPT,   /* ΥץΥȥ(Helpɽ)                         */
    ZIPC    /* ΥץΥ(Helpɽ)                         */
  };
    
/*----------------------------------------------------------------------------*/
/* plain text                                                                 */
/*----------------------------------------------------------------------------*/
  MssOptFLG optTXT={
    OFLG,   /* ץ󥿥                                             */
    "t",    /* (ʣʸԲ)                                   */
    0,      /* ǥե(Ūˤ0) onˤȤ1ˤ          */
    TXTT,   /* ΥץΥȥ(Helpɽ)                         */
    TXTC    /* ΥץΥ(Helpɽ)                         */
  };

/*----------------------------------------------------------------------------*/
/* եѥǥ쥯ȥ̾                                             */
/*----------------------------------------------------------------------------*/
  MssOptSTR optTMP={
    OSTR,   /* ץ󥿥                                             */
    "T",    /* (ʣʸԲ)                                   */
    0,      /* 0:ץ, 1:ɬ, 2:XMLtableǤΤɬ(txtǤ̵)      */
    MssTempDir, /* ǥե                                               */
    1,      /* ʸκǾĹ                                               */
    MssFileNameMaxLen,  /* ʸκĹ                                   */
    TMPT,   /* ΥץΥȥ(Helpɽ)                         */
    TMPC    /* ΥץΥ(Helpɽ)                         */
  };
    
/*----------------------------------------------------------------------------*/
/* ץޤȤ                                                       */
/*----------------------------------------------------------------------------*/
  void *opt[]={&optDIR,&optFRC,&optINF,&optZIP,&optTXT,&optTMP,NULL};
    
/*============================================================================*/
/* ѿ                                                             */
/*============================================================================*/
  struct mssHeader    *hdi; /*ϥե<head>Ǽ¤*/
  struct mssHeader    *hdo; /*ϥե<head>Ǽ¤*/
  struct mssFPR       *fpr; /*ϥե빽¤                */
  struct mssFPW       *fpw=NULL; /*ϥե빽¤                */
  struct mssFldRecDbl *frd; /*-ԥХåե¤           */
  struct mssFields    *sf;  /*ȹܹ¤                  */
  int sorted;               /*Ⱥѥå                */

  struct Path *path;
  char pathName[MssFileNameMaxLen];
  int count;

/*----------------------------------------------------------------------------*/
/*                                                                      */
/*----------------------------------------------------------------------------*/
  mssInit(argc,argv,&comHelp);       /* ʥʤɤν              */
  mssHelpDoc(opt,&comHelp,argc,argv);/* إ                                */
  mssSetOption(opt,argc,argv);       /* ޥɥץ              */
  fpr=mssOpenFPR(optINF.str,4);      /* ϥե륪ץ                  */
  hdi=mssReadHeader(fpr);            /* إåɤ߹                      */

  path=setPath(optDIR.str,hdi);   /*path򥻥å*/

  /* optKey򤳤Ƕ*/
  optKEY.set=1;
  optKEY.str=mssStrdup(path->optStr);
  optKEY.cnt=0;
  optKEY.nam=mssTokByChr(optKEY.str,',',&optKEY.cnt,1); /*,ˤȡʬ*/
  optKEY.diffSame=0;
  mssSetOptKey(&optKEY, hdi);        /* -k ܤإåܤ˴ϢŤ     */

  /*ȹܤκ*/
  sf=mssInitFields();
  mssAddFieldsByFields(sf,optKEY.flds); /* -k ܤ򥽡ȹܤȤƥå    */
  mssSetFieldsSortPriority(sf);         /* ֹͥϿˤդ   */
  sorted=mssChkSorted(sf,hdi);          /* ȺѤå                 */

/*----------------------------------------------------------------------------*/
/*ϥإåκȽ                                                    */
/*----------------------------------------------------------------------------*/
  /*ϥإåν(ȥΥԡ)*/
  hdo=mssInitCpyHeader(hdi);
    
  /*ϥإåܤɲ*/
  mssAddFieldsByFields(hdo->flds,hdi->flds);                                
    
  /*ȤɬפʤsfΥȾȿ*/                        
  if(!sorted){
    mssSetFieldsSort(hdo->flds,sf);
  }

/*----------------------------------------------------------------------------*/
/*ᥤ롼                                                              */
/*----------------------------------------------------------------------------*/
  /*ȤɬѤʤХȤƥȺѤߥեȤƥץ*/
  if(!sorted)
    fpr=mssReopenFPRsort(fpr,4,sf,hdi->flds->cnt,optTMP.str);

  frd=mssInitFRD(hdi->flds->cnt);
  while( EOF!=mssReadFRD(fpr,frd) ){

    if(mssGV.inCnt==0){
      setPathName(pathName,path,frd,&optKEY);
      fpw=mssOpenFPW(pathName,optZIP.set,optFRC.set);
      mssWriteHeader(hdo, fpw);
    }

    /*֥쥤ν*/
    if(mssKeyBreak(frd, &optKEY)){
      mssWriteFooter(fpw);    /*եåν*/
      mssCloseFPW(fpw);                /*ϥեΥ*/
      if(frd->eof) break;
      setPathName(pathName,path,frd,&optKEY);
      fpw=mssOpenFPW(pathName,optZIP.set,optFRC.set);
      mssWriteHeader(hdo, fpw);
    }

    mssGV.inCnt++;
    count++;

    /*̾Ԥν*/
    mssWriteFld(frd->pnt[frd->new],frd->fldCnt,"\n",fpw);
    mssGV.outCnt++;
  }
  mssFreeFRD(frd);
  mssFreeFields(path->flds);
  mssFree(path->optStr);
  mssFree(path);
  mssFree(optKEY.str);
  mssFree(*optKEY.nam);
  mssFree(optKEY.nam);

/*----------------------------------------------------------------------------*/
/*եå&λ                                                       */
/*----------------------------------------------------------------------------*/
  mssCloseFPR(fpr);       /*ϥեΥ      */
  mssFreeFields(sf);      /* ȹܹ¤Τΰ賫 */
  mssFreeHeader(hdi);     /* ϥإåΰ賫         */
  mssFreeHeader(hdo);     /* ϥإåΰ賫         */
  mssFreeOption(opt);     /* ץΰ賫         */
  mssShowEndMsg();        /* λå             */
  mssEnd(mssExitSuccess); /* λ                       */
  return(0);              /* to avoid warning message   */
}
